summary refs log tree commit diff
path: root/crypto/src/asn1/DerPrintableString.cs
diff options
context:
space:
mode:
authorPeter Dettman <peter.dettman@bouncycastle.org>2021-11-18 13:46:18 +0700
committerPeter Dettman <peter.dettman@bouncycastle.org>2021-11-18 13:46:18 +0700
commit4fccb8f490eefe7181558b2c3376ab23c33632ee (patch)
treefe8c1877bb5567cb4823823af77e5c8a38423db8 /crypto/src/asn1/DerPrintableString.cs
parentASN.1: Staged encoding (diff)
downloadBouncyCastle.NET-ed25519-4fccb8f490eefe7181558b2c3376ab23c33632ee.tar.xz
ASN.1: Port of bc-java TYPE instances
- we use Meta.Instance here due to syntax restrictions
- also reworked some ASN.1 string types
Diffstat (limited to 'crypto/src/asn1/DerPrintableString.cs')
-rw-r--r--crypto/src/asn1/DerPrintableString.cs126
1 files changed, 74 insertions, 52 deletions
diff --git a/crypto/src/asn1/DerPrintableString.cs b/crypto/src/asn1/DerPrintableString.cs
index 30358e219..3c44a2d52 100644
--- a/crypto/src/asn1/DerPrintableString.cs
+++ b/crypto/src/asn1/DerPrintableString.cs
@@ -1,4 +1,5 @@
 using System;
+using System.IO;
 
 using Org.BouncyCastle.Utilities;
 
@@ -10,61 +11,65 @@ namespace Org.BouncyCastle.Asn1
     public class DerPrintableString
         : DerStringBase
     {
-        private readonly string str;
+        internal class Meta : Asn1UniversalType
+        {
+            internal static readonly Asn1UniversalType Instance = new Meta();
+
+            private Meta() : base(typeof(DerPrintableString), Asn1Tags.PrintableString) {}
+
+            internal override Asn1Object FromImplicitPrimitive(DerOctetString octetString)
+            {
+                return CreatePrimitive(octetString.GetOctets());
+            }
+        }
 
 		/**
          * return a printable string from the passed in object.
          *
          * @exception ArgumentException if the object cannot be converted.
          */
-        public static DerPrintableString GetInstance(
-            object obj)
+        public static DerPrintableString GetInstance(object obj)
         {
             if (obj == null || obj is DerPrintableString)
             {
                 return (DerPrintableString)obj;
             }
+            else if (obj is IAsn1Convertible)
+            {
+                Asn1Object asn1Object = ((IAsn1Convertible)obj).ToAsn1Object();
+                if (asn1Object is DerPrintableString)
+                    return (DerPrintableString)asn1Object;
+            }
+            else if (obj is byte[])
+            {
+                try
+                {
+                    return (DerPrintableString)Meta.Instance.FromByteArray((byte[])obj);
+                }
+                catch (IOException e)
+                {
+                    throw new ArgumentException("failed to construct printable string from byte[]: " + e.Message);
+                }
+            }
 
             throw new ArgumentException("illegal object in GetInstance: " + Platform.GetTypeName(obj));
         }
 
         /**
-         * return a Printable string from a tagged object.
+         * return a printable string from a tagged object.
          *
          * @param obj the tagged object holding the object we want
          * @param explicitly true if the object is meant to be explicitly
          *              tagged false otherwise.
          * @exception ArgumentException if the tagged object cannot
          *               be converted.
          */
-        public static DerPrintableString GetInstance(
-            Asn1TaggedObject	obj,
-            bool				isExplicit)
+        public static DerPrintableString GetInstance(Asn1TaggedObject taggedObject, bool declaredExplicit)
         {
-			Asn1Object o = obj.GetObject();
-
-			if (isExplicit || o is DerPrintableString)
-			{
-				return GetInstance(o);
-			}
-
-			return new DerPrintableString(Asn1OctetString.GetInstance(o).GetOctets());
+            return (DerPrintableString)Meta.Instance.GetContextInstance(taggedObject, declaredExplicit);
         }
 
-        /**
-         * basic constructor - byte encoded string.
-         */
-        public DerPrintableString(
-            byte[] str)
-            : this(Strings.FromAsciiByteArray(str), false)
-        {
-        }
+        private readonly byte[] m_contents;
 
-		/**
-		 * basic constructor - this does not validate the string
-		 */
-		public DerPrintableString(
-			string str)
+		public DerPrintableString(string str)
 			: this(str, false)
 		{
 		}
@@ -77,57 +82,69 @@ namespace Org.BouncyCastle.Asn1
 		* @throws ArgumentException if validate is true and the string
 		* contains characters that should not be in a PrintableString.
 		*/
-		public DerPrintableString(
-			string	str,
-			bool	validate)
+		public DerPrintableString(string str, bool validate)
 		{
 			if (str == null)
 				throw new ArgumentNullException("str");
 			if (validate && !IsPrintableString(str))
 				throw new ArgumentException("string contains illegal characters", "str");
 
-			this.str = str;
+            m_contents = Strings.ToAsciiByteArray(str);
 		}
 
-		public override string GetString()
+        public DerPrintableString(byte[] contents)
+            : this(contents, true)
+        {
+        }
+
+        internal DerPrintableString(byte[] contents, bool clone)
+        {
+            if (null == contents)
+                throw new ArgumentNullException("contents");
+
+            m_contents = clone ? Arrays.Clone(contents) : contents;
+        }
+
+        public override string GetString()
         {
-            return str;
+            return Strings.FromAsciiByteArray(m_contents);
         }
 
-		public byte[] GetOctets()
+        public byte[] GetOctets()
         {
-            return Strings.ToAsciiByteArray(str);
+            return Arrays.Clone(m_contents);
         }
 
         internal override IAsn1Encoding GetEncoding(int encoding)
         {
-            return new PrimitiveEncoding(Asn1Tags.Universal, Asn1Tags.PrintableString, GetOctets());
+            return new PrimitiveEncoding(Asn1Tags.Universal, Asn1Tags.PrintableString, m_contents);
         }
 
         internal override IAsn1Encoding GetEncodingImplicit(int encoding, int tagClass, int tagNo)
         {
-            return new PrimitiveEncoding(tagClass, tagNo, GetOctets());
+            return new PrimitiveEncoding(tagClass, tagNo, m_contents);
         }
 
         protected override bool Asn1Equals(
 			Asn1Object asn1Object)
 		{
-			DerPrintableString other = asn1Object as DerPrintableString;
-
-			if (other == null)
-				return false;
+            DerPrintableString that = asn1Object as DerPrintableString;
+            return null != that
+                && Arrays.AreEqual(this.m_contents, that.m_contents);
+        }
 
-			return this.str.Equals(other.str);
+        protected override int Asn1GetHashCode()
+        {
+            return Arrays.GetHashCode(m_contents);
         }
 
-		/**
+        /**
 		 * return true if the passed in String can be represented without
 		 * loss as a PrintableString, false otherwise.
 		 *
 		 * @return true if in printable set, false otherwise.
 		 */
-		public static bool IsPrintableString(
-			string str)
+        public static bool IsPrintableString(string str)
 		{
 			foreach (char ch in str)
 			{
@@ -162,5 +179,10 @@ namespace Org.BouncyCastle.Asn1
 
 			return true;
 		}
-	}
+
+        internal static DerPrintableString CreatePrimitive(byte[] contents)
+        {
+            return new DerPrintableString(contents, false);
+        }
+    }
 }
@@ -10,61 +11,65 @@ namespace Org.BouncyCastle.Asn1
          *
-         * @param obj the tagged object holding the object we want
-         * @param explicitly true if the object is meant to be explicitly
-         *              tagged false otherwise.
-         * @exception ArgumentException if the tagged object cannot
-         *               be converted.
+         * @param taggedObject the tagged object holding the object we want
+         * @param declaredExplicit true if the object is meant to be explicitly tagged false otherwise.
+         * @exception ArgumentException if the tagged object cannot be converted.
          */